<?php
if (!defined('ABSPATH')) die('-1');

if (!class_exists("WD_ASP_Search_Handler")) {
    /**
     * Class WD_ASP_Search_Handler
     *
     * This is the ajax search handler class
     *
     * @class         WD_ASP_Search_Handler
     * @version       1.0
     * @package       AjaxSearchPro/Classes/Ajax
     * @category      Class
     * @author        Ernest Marcinko
     */
    class WD_ASP_Search_Handler extends WD_ASP_Handler_Abstract {


        /**
         * Oversees and handles the search request
         *
         * @param bool $dontGroup
         * @return array|mixed|void
         */
        function handle($dontGroup = false) {

            $perf_options = get_option('asp_performance');

            if (w_isset_def($perf_options['enabled'], 1)) {
                $performance = new wpd_Performance('asp_performance_stats');
                $performance->start_measuring();
            }

            $s = $_POST['aspp'];
            $s = apply_filters('asp_search_phrase_before_cleaning', $s);

            $s = stripcslashes($s);
            $s = trim($s);
            $s = preg_replace('/\s+/', ' ', $s);

            $s = apply_filters('asp_search_phrase_after_cleaning', $s);

            $id = (int)$_POST['asid'];
            //$sd = $asp_globals->instances[$id]['data'];
            $instance = wd_asp()->instances->get($id);
            $sd = &$instance['data'];

            // Disabled compact layout if the box is hidden anyways
            if ( $sd['box_sett_hide_box'] == 1 ) {
                $sd['box_compact_layout'] = 0;
                $sd['frontend_search_settings_visible'] = 1;
                $sd['show_frontend_search_settings'] = 1;
                $sd['frontend_search_settings_position'] = "block";
                $sd['resultsposition'] = "block";
                $sd['charcount'] = 0;
                $sd['trigger_on_facet'] = 1;
            }

            if (get_option("asp_stat") == 1)
                asp_statistics::addKeyword($id, $s);

            $searchController = new asp_searchController(array(
                "phrase"    => $s,
                "id"        => $id,
                "instance"  => $instance,
                "dontGroup" => $dontGroup
            ));

            $results = $searchController->search();

            if (count($results) <= 0 && $sd['keywordsuggestions']) {
                $results = $searchController->kwSuggestions();
            } else if (count($results > 0)) {
                $results = apply_filters('asp_only_non_keyword_results', $results);
            }

            $results = apply_filters('asp_results', $results, $id);

            do_action('asp_after_search', $s, $results, $id);

            if (w_isset_def($perf_options['enabled'], 1)) {
                $performance->stop_measuring();
                //$performance->dump_data();
            }

            // If preview, we need the details
            if ( current_user_can("activate_plugins") && isset($_POST['asp_preview_options']) ) {
                require_once(ASP_PATH . "backend" . DIRECTORY_SEPARATOR . "settings" . DIRECTORY_SEPARATOR . "types.inc.php");
                $preview_params = unserialize( base64_decode($_POST['asp_preview_options']) );
                $preview_params = wpdreams_parse_params($preview_params);
                $preview_params = wp_parse_args( $preview_params, $sd );
                $html_results = asp_generate_html_results($results, $preview_params, $preview_params['resultstype']);
            } else {
                $html_results = asp_generate_html_results($results, $sd, $sd['resultstype']);
            }

            $searchController->setResCache($html_results);

            // Override from hooks
            if (isset($_POST['asp_get_as_array'])) {
                return $results;
            }
            /* Clear output buffer, possible warnings */
            print "!!ASPSTART!!";
            //var_dump($results);die();
            //print_r(json_encode($results));
            print_r($html_results);
            print "!!ASPEND!!";
            die();
        }

        // ------------------------------------------------------------
        //   ---------------- SINGLETON SPECIFIC --------------------
        // ------------------------------------------------------------
        public static function getInstance() {
            if ( ! ( self::$_instance instanceof self ) ) {
                self::$_instance = new self();
            }

            return self::$_instance;
        }
    }
}