<?php
if (!defined('ABSPATH')) die('-1');

if (!class_exists("WD_ASP_Menu")) {
    /**
     * Class WD_ASP_Menu
     *
     * Menu handler for Ajax Search Pro plugin. This class encapsulates the menu elements as well.
     *
     * @class         WD_ASP_Menu
     * @version       1.0
     * @package       AjaxSearchPro/Classes/Core
     * @category      Class
     * @author        Ernest Marcinko
     */
    class WD_ASP_Menu {

        /**
         * Holds the main menu item
         *
         * @var array the main menu
         */
        private static $main_menu = array(
            "title" => "Ajax Search Pro",
            "slug" => "/backend/settings.php",
            "position" => "207.9",
            "icon_url" => "icon.png"
        );

        /**
         * Submenu titles and slugs
         *
         * @var array
         */
        private static $submenu_items = array(
            array(
                "title" => "Index Table",
                "slug" => "/backend/index_table.php"
            ),
            array(
                "title" => "Priorities",
                "slug" => "/backend/priorities.php"
            ),
            array(
                "title" => "Search Statistics",
                "slug" => "/backend/statistics.php"
            ),
            array(
                "title" => "Analytics Integration",
                "slug" => "/backend/analytics.php"
            ),
            array(
                "title" => "Cache Settings",
                "slug" => "/backend/cache_settings.php"
            ),
            array(
                "title" => "Performance tracking",
                "slug" => "/backend/performance.php"
            ),
            array(
                "title" => "Compatibility Settings",
                "slug" => "/backend/compatibility_settings.php"
            ),
            array(
                "title" => "Export/Import",
                "slug" => "/backend/export_import.php"
            ),
            array(
                "title" => "Help & Updates",
                "slug" => "/backend/updates_help.php"
            )
        );

        /**
         * Runs the menu registration process
         */
        public static function register() {

            $capability = ASP_DEMO == 1 ? 'read' : 'manage_options';

            add_menu_page(
                self::$main_menu['title'],
                self::$main_menu['title'],
                $capability,
                ASP_DIR . self::$main_menu['slug'],
                '',
                ASP_URL . self::$main_menu['icon_url'],
                self::$main_menu['position']
            );

            foreach (self::$submenu_items as $submenu) {
                add_submenu_page(
                    ASP_DIR . self::$main_menu['slug'],
                    self::$main_menu['title'],
                    $submenu['title'],
                    $capability,
                    ASP_DIR . $submenu['slug']
                );

            }

        }

        /**
         * Method to obtain the menu pages for context checking
         *
         * @return array
         */
        public static function getMenuPages() {
            $ret = array();

            $ret[] = ASP_DIR . self::$main_menu['slug'];

            foreach (self::$submenu_items as $menu)
                $ret[] = ASP_DIR . $menu['slug'];

            return $ret;
        }

    }
}