<?php
if (!defined('ABSPATH')) die('-1');

if (!class_exists("WD_ASP_SearchOverride_Filter")) {
    /**
     * Class WD_ASP_SearchOverride_Filter
     *
     * Handles search override filters
     *
     * @class         WD_ASP_SearchOverride_Filter
     * @version       1.0
     * @package       AjaxSearchPro/Classes/Filters
     * @category      Class
     * @author        Ernest Marcinko
     */
    class WD_ASP_SearchOverride_Filter extends WD_ASP_Filter_Abstract {

        function handle() {}

        function override($posts, $wp_query) {

            // Not working on paginated results, multisite
            /*if (!$wp_query->is_search())
                return $posts;*/

            // Is this a search query
            if (empty($wp_query->query_vars['s'])) {
                return $posts;
            }

            $cookie_cleared = false;

            // Is this just a NEW regular search?
            if (
            !isset($_GET['asp_active'])
            ) {
                // Then clear the cookie
                setcookie("asp_cookie", "", time()-3600);
                $cookie_cleared = true;
            }

            if (isset($_POST['p_asp_data']) && $_POST['p_asp_data'] != '') {
                $_method = &$_POST;
                parse_str($_method['p_asp_data'], $s_data);
                $_POST['asid'] = $_POST['p_asid'];

                // this is a new POST search, so set the cookies
                setcookie("asp_cookie", base64_encode(serialize(array(
                    "asp_data" => $s_data,
                    "asid" => $_POST['p_asid']
                ))) );
            } else if (isset($_GET['asp_data']) && $_GET['asp_data'] != '') {
                $_method = &$_GET;
                parse_str(base64_decode($_method['asp_data']), $s_data);
                $_POST['asid'] = $s_data['asid'];

                // this is a new GET search, so set the cookies
                setcookie("asp_cookie", base64_encode(serialize(array(
                    "asp_data" => $s_data,
                    "asid" => $_POST['p_asid']
                ))) );
            } else {
                if ( isset($_COOKIE["asp_cookie"]) && !$cookie_cleared ) {
                    $c_val = unserialize( base64_decode($_COOKIE["asp_cookie"]) );
                    $s_data = $c_val['asp_data'];
                    $_POST['asid'] = $c_val['asid'];
                } else {
                    return $posts;
                }
            }

            $_POST['options'] = $s_data;
            $_POST['options']['non_ajax_search'] = true;
            $_POST['aspp'] = $_GET['s'];
            $_POST['asp_get_as_array'] = 1;

            //require_once(ASP_PATH . '/search.php');
            //$res = ajaxsearchpro_search(true);
            $o = WD_ASP_Search_Handler::getInstance();
            $res = $o->handle( true );

            // The get_query_var() is malfunctioning in some cases!!! use $_GET['paged']
            //$paged = (get_query_var('paged') != 0) ? get_query_var('paged') : 1;
            if ( isset($_GET['paged']) ) {
                $paged = $_GET['paged'];
            } else if ( isset($wp_query->query_vars['paged']) ) {
                $paged = $wp_query->query_vars['paged'];
            } else {
                $paged = 1;
            }

            $paged = $paged <= 0 ? 1 : $paged;

            $posts_per_page = (int)get_option('posts_per_page');

            // Get and convert the results needed
            $n_posts = asp_results_to_wp_obj( $res, ( $paged - 1 ) * $posts_per_page, $posts_per_page );

            $wp_query->found_posts = count($res);
            if (($wp_query->found_posts / $posts_per_page) > 1)
                $wp_query->max_num_pages = ceil($wp_query->found_posts / $posts_per_page);
            else
                $wp_query->max_num_pages = 0;

            return $n_posts;
        }

        function fixUrls( $url, $post, $leavename ) {
            if (isset($post->asp_guid))
                return $post->asp_guid;
            return $url;
        }

        // ------------------------------------------------------------
        //   ---------------- SINGLETON SPECIFIC --------------------
        // ------------------------------------------------------------
        public static function getInstance() {
            if ( ! ( self::$_instance instanceof self ) ) {
                self::$_instance = new self();
            }

            return self::$_instance;
        }
    }
}