<?php
/* Prevent direct access */
defined('ABSPATH') or die("You can't access this file directly.");

if (!class_exists('wpdreams_searchComments')) {
    /**
     * Comment search class
     *
     * @class       wpdreams_searchComments
     * @version     1.0
     * @package     AjaxSearchPro/Abstracts
     * @category    Class
     * @author      Ernest Marcinko
     */
    class wpdreams_searchComments extends wpdreams_searchContent
    {

        /**
         * The search function
         *
         * @return array of results
         */
        protected function do_search() {
            global $wpdb;
            $commentsresults = array();

            $options = $this->options;
            $searchData = $this->searchData;
            $s = $this->s;
            $_s = $this->_s;

            // Prefixes and suffixes
            $pre_field = '';
            $suf_field = '';
            $pre_like = '';
            $suf_like = '';

            $kw_logic = $searchData['keyword_logic'];

            if ($options['set_incomments'] && count($_s) > 0) {


                // ------------------------ Categories/taxonomies ----------------------
                $term_query = $this->build_term_query( $wpdb->comments.".comment_post_ID" );
                // ---------------------------------------------------------------------

                /*------------- Custom Fields with Custom selectors -------------*/
                $cf_select = $this->build_cff_query( $wpdb->comments.".comment_post_ID" );
                /*---------------------------------------------------------------*/

                /*----------------------- Date filtering ------------------------*/
                $date_query = "";
                $date_query_parts = $this->get_date_query_parts( "ddpp" );

                if ( count($date_query_parts) > 0 )
                    $date_query = " AND EXISTS( SELECT 1 FROM $wpdb->posts as ddpp WHERE " . implode(" AND ", $date_query_parts) . " AND ddpp.ID = $wpdb->comments.comment_post_ID) ";
                /*---------------------------------------------------------------*/

                /*---------------------- Content query --------------------------*/

                $like_part = "";
                $relevance_part = "1";
                $words = $options['set_exactonly'] == 1 ? array($s) : $_s;
                if ( $s != "" ) {
                    if ($kw_logic == 'or' || $kw_logic == 'and') {
                        $op = strtoupper($kw_logic);
                        if (count($_s) > 0) {
                            $_like = implode("%'$suf_like " . $op . " " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'%", $words);
                        } else {
                            $_like = $s;
                        }
                        $like_part = "AND ( " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'%" . $_like . "%'$suf_like )";
                    } else {
                        $_like = array();
                        $op = $kw_logic == 'andex' ? 'AND' : 'OR';
                        foreach ($words as $word) {
                            $_like[] = "
                       (" . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'% " . $word . " %'$suf_like
                    OR  " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'" . $word . " %'$suf_like
                    OR  " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE $pre_like'% " . $word . "'$suf_like
                    OR  " . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " = '" . $word . "')";
                        }
                        $like_part = "AND (" . implode(' ' . $op . ' ', $_like) . ")";
                    }

                    if (count($_s) > 0) {
                        $relevance_part = "(case when
                (" . $pre_field . $wpdb->comments . ".comment_content" . $suf_field . " LIKE '%" . $_s[0] . "%')
                 then $searchData[contentweight] else 0 end)";
                    }
                }

                /*---------------------------------------------------------------*/

                $orderby_primary    = str_replace( "post_", "",  $searchData['orderby_primary'] );
                $orderby_secondary  = str_replace( "post_", "",  $searchData['orderby'] );

                if (isset($options['non_ajax_search']))
                    $limit = 500;
                else
                    $limit = $searchData['maxresults'];

                $querystr = "
      		SELECT 
            $wpdb->comments.comment_ID as id,
            $this->c_blogid as blogid,
            $wpdb->comments.comment_post_ID as post_id,
            $wpdb->comments.user_id as user_id,
            $wpdb->comments.comment_content as title,
            $wpdb->comments.comment_content as content,
            'comment' as content_type,
            $wpdb->comments.comment_date as date,
            $relevance_part as relevance
      		FROM $wpdb->comments
      		WHERE
          ($wpdb->comments.comment_approved=1)
          $term_query
          AND $cf_select
          $date_query
          $like_part
      		ORDER BY $orderby_primary, $orderby_secondary
      		LIMIT " . $limit;
                //var_dump($querystr);
                $commentsresults = $wpdb->get_results($querystr, OBJECT);

            }
            $this->results = &$commentsresults;
            return $commentsresults;
        }

        function post_process() {
            $r = &$this->results;

            if (is_array($r)) {
                foreach ($r as $k => $v) {
                    $r[$k]->link = get_comment_link($v->id);
                    $r[$k]->author = get_comment_author($v->id);

                    $r[$k]->title = wd_substr_at_word($r[$k]->content, 40) . "...";

                }
            }
        }

    }
}