<?php
/* Prevent direct access */
defined('ABSPATH') or die("You can't access this file directly.");

if (!class_exists('asp_searchIndexTable')) {
	/**
	 * Index table search class
	 *
	 * @class       asp_searchIndexTable
	 * @version     1.0
	 * @package     AjaxSearchPro/Classes
	 * @category    Class
	 * @author      Ernest Marcinko
	 */
	class asp_searchIndexTable extends wpdreams_searchContent {

		/**
		 * @var array of query parts
		 */
		protected $parts = array();
		/**
		 * @var array of custom field query parts
		 */
		protected $cf_parts = array();
		/**
		 * @var int the remaining limit (number of items to look for)
		 */
		protected $remaining_limit;
		/**
		 * @var string the final search query
		 */
		protected $query;

		/**
		 * @var array results from the index table
		 */
		protected $raw_results = array();

		/**
		 * Content search function
		 *
		 * @return array|string
		 */
		protected function do_search() {
			global $wpdb;
			global $q_config;

			$options = $this->options;
			$comp_options = get_option('asp_compatibility');
			$searchId = $this->searchId;
			$searchData = $this->searchData;
            $sd = &$this->searchData;
			$current_blog_id = get_current_blog_id();

            $aspdb = wd_asp()->tables;

			// General variables
			$parts = array();
			$relevance_parts = array();
			$types = array();
			$post_types = "(1)";
			$term_query = "(1)";
			$post_statuses = "(1)";

			$kw_logic = $searchData['keyword_logic'];
			$q_config['language'] = $options['qtranslate_lang'];

			$s = $this->s; // full keyword
			$_s = $this->_s; // array of keywords

            $sr = $this->sr; // full reversed keyword
            $_sr = $this->_sr; // array of reversed keywords

			/**
			 * Determine if the priorities table should be used or not.
			 */
            $priority_rows   = (int) $wpdb->get_var( "SELECT 1 FROM $aspdb->priorities LIMIT 1" );
			$priority_select = $wpdb->num_rows > 0 ? "
	        IFNULL((
            	SELECT
	            aspp.priority
	            FROM $aspdb->priorities as aspp
	            WHERE aspp.post_id = asp_index.doc AND aspp.blog_id = " . get_current_blog_id() . "
            ), 100)
	        " : 100;


			/*------------------------- Statuses ----------------------------*/
			// Removed - it is already considered at index generation
			/*---------------------------------------------------------------*/

			/*----------------------- Gather Types --------------------------*/
			$page_q = "";
			if ($options['set_inposts'] == 1)
				$types[] = "post";
			if ($options['set_inpages']) {
				if ( $searchData['exclude_page_parent_child'] != '' )
					// .. page exists thats parent isnt on the list
					$page_q = " OR (asp_index.post_type = 'page' AND
						EXISTS (SELECT ID FROM $wpdb->posts xxp WHERE
							xxp.ID = asp_index.doc AND
							xxp.post_parent NOT IN (".str_replace('|', ',', $searchData['exclude_page_parent_child']).") AND
							xxp.ID NOT IN (".str_replace('|', ',', $searchData['exclude_page_parent_child']).")
						)
					)";
				else
					$types[] = "page";
			}
			if (isset($options['customset']) && count($options['customset']) > 0)
				$types = array_merge($types, $options['customset']);

			// If no post types selected, well then return
			if (count($types) < 1 && $page_q == "") {
				return '';
			} else {
				$words = implode("','", $types);
				$post_types = "(asp_index.post_type IN ('$words') $page_q)";
			}
			/*---------------------------------------------------------------*/


            // ------------------------ Categories/taxonomies ----------------------
            $term_query = $this->build_term_query( "asp_index.doc" );
            // ---------------------------------------------------------------------


            // ------------------------- TAGS QUERY --------------------------------
            $tag_query = $this->build_tag_query( "asp_index.doc", "asp_index.post_type" );
            // ---------------------------------------------------------------------


			/*------------ ttids in the main SELECT, we might not need it ---------*/
			// ttid is only used if grouping by category or filtering by category is active
			$term_select = '""';
			if ($searchData['groupby'] == 1) {
				$term_select = "(SELECT DISTINCT CONCAT('--', GROUP_CONCAT( $wpdb->term_relationships.term_taxonomy_id SEPARATOR '----' ), '--')
                FROM $wpdb->term_relationships
                WHERE ($wpdb->term_relationships.object_id = asp_index.doc) )";
			}
			// ---------------------------------------------------------------------


            /*------------- Custom Fields with Custom selectors -------------*/
            $cf_select = $this->build_cff_query( "asp_index.doc" );
            /*---------------------------------------------------------------*/


			/*------------------------ Exclude id's -------------------------*/
			if (isset($searchData['excludeposts']) && $searchData['excludeposts'] != "") {
				$exclude_posts = "(asp_index.doc NOT IN (" . $searchData['excludeposts'] . "))";
			} else {
				$exclude_posts = "(1)";
			}
			/*---------------------------------------------------------------*/

			/*------------------------ Term JOIN -------------------------*/
			// No need, this should be indexed...
			/*---------------------------------------------------------------*/

            /*------------------------- WPML filter -------------------------*/
            $wpml_query = "(1)";
            if ( isset($options['wpml_lang'])
                && $options['wpml_lang'] != ""
                && $searchData['wpml_compatibility'] == 1
            ) {
                global $sitepress;
                $site_lang_selected = false;

                if ( is_object($sitepress) && method_exists($sitepress, 'get_default_language') ) {
                    $site_lang_selected = $sitepress->get_default_language() == $options['wpml_lang'] ? true : false;
                }

                $wpml_query = "asp_index.lang = '" . $this->escape($options['wpml_lang']) . "'";

                /**
                 * Imported or some custom post types might have missing translations for the site default language.
                 * If the user currently searches on the default language, empty translation string is allowed.
                 */
                if ($site_lang_selected)
                    $wpml_query .= " OR asp_index.lang = ''";
            }

            /*---------------------------------------------------------------*/

            /*----------------------- Date filtering ------------------------*/
            $date_query = "";
            $date_query_parts = $this->get_date_query_parts( "ddpp" );

            if ( count($date_query_parts) > 0 )
                $date_query = " AND EXISTS( SELECT 1 FROM $wpdb->posts as ddpp WHERE " . implode(" AND ", $date_query_parts) . " AND ddpp.ID = asp_index.doc) ";
            /*---------------------------------------------------------------*/

			/*----------------------- Optimal LIMIT -------------------------*/
            // If custom field filtering is enabled, we need to be careful with the limiting
            if ( count($this->cf_parts) > 0) {
                $limit = floor( $searchData['maxresults'] ) * 200;
            } else {
                /**
                 * Instead of the flippy-floppy calculations, just set this to 5k
                 * This is the pool size for each result set. It is important when using
                 * AND and similar logics.
                 * DO NOT set it too high, as PHP might run out of memory.
                 */
                $limit = 5000;
            }
			/*---------------------------------------------------------------*/

			/*---------------------- Blog switching? ------------------------*/
			if (isset($options['switch_on_preprocess']))
				$blog_query = "asp_index.blogid IN (".implode(',', $searchData['selected-blogs']) . ")";
			else
				$blog_query = "asp_index.blogid = ".$current_blog_id;
			/*---------------------------------------------------------------*/

            /*---------------------- Relevance Values -----------------------*/
            $rel_val_title = $searchData["it_title_weight"];
            $rel_val_content = $searchData["it_content_weight"];
            $rel_val_excerpt = $searchData["it_excerpt_weight"];
            $rel_val_terms = $searchData["it_terms_weight"];
            $rel_val_cf = $searchData["it_cf_weight"];
            $rel_val_author = $searchData["it_author_weight"];
            /*---------------------------------------------------------------*/

            /*-------------- Additional Query parts by Filters --------------*/
            /**
             * Use these filters to add additional parts to the select, join or where
             * parts of the search query.
             */
            $add_select = apply_filters('asp_it_query_add_select', '');
            $add_join = apply_filters('asp_it_query_add_join', '');
            $add_where = apply_filters('asp_it_query_add_where', '');
            /*---------------------------------------------------------------*/

			/**
			 * This is the main query.
			 *
			 * The ttid field is a bit tricky as the term_taxonomy_id doesn't always equal term_id,
			 * so we need the LEFT JOINS :(
			 */
            $this->ordering['primary'] = w_isset_def( $searchData['orderby_primary'], "relevance DESC" );
            $this->ordering['secondary'] = w_isset_def( $searchData['orderby'], "post_date DESC" );

            $_primary_field = explode(" ", $this->ordering['primary']);
            $this->ordering['primary_field'] = $_primary_field[0];

			$this->query = "
    		SELECT
    		    $add_select
                asp_index.doc as id,
                asp_index.blogid as blogid,
                'pagepost' as content_type,
                $term_select as ttid,
                $priority_select AS priority,
                asp_index.post_type AS post_type,
                (
                     asp_index.title * $rel_val_title * {rmod} +
                     asp_index.content * $rel_val_content * {rmod}  +
                     asp_index.excerpt * $rel_val_excerpt * {rmod}  +
                     asp_index.comment * $rel_val_terms * {rmod}  +
                     asp_index.tag * $rel_val_terms * {rmod}  +
                     asp_index.customfield * $rel_val_cf * {rmod}  +
                     asp_index.author * $rel_val_author * {rmod}
                )as relevance
            FROM
              $aspdb->index as asp_index
              $add_join
            WHERE
                    {like_query}
                AND $post_types
                AND $blog_query
				AND ( $wpml_query )
                $term_query
                $tag_query
                AND $cf_select
                AND $post_statuses
                AND $exclude_posts
                $date_query
                $add_where
            LIMIT $limit";

			$queries = array();
			$results_arr = array();

			//$words = $options['set_exactonly'] == 1 ? array($s) : $_s;
            $words = $_s;

			if ($kw_logic == "orex") {
                $rmod = 1;
				$like_query = "asp_index.term = '" . implode( "' OR asp_index.term = '", $words ) . "'";
                $queries[]  = str_replace( array('{like_query}', '{rmod}'), array($like_query, $rmod), $this->query );
			} else if ($kw_logic == "andex") {
				foreach ( $words as $wk => $word ) {
                    $rmod = 10 - ( $wk * 8 ) < 1 ? 1 : 10 - ( $wk * 8 );

					$like_query = "asp_index.term = '$word'";
                    $queries[]  = str_replace( array('{like_query}', '{rmod}'), array($like_query, $rmod), $this->query );
				}
			} else {
				foreach ( $words as $wk => $word ) {
                    $rmod = 10 - ( $wk * 8 ) < 1 ? 1 : 10 - ( $wk * 8 );

					$like_query = "asp_index.term LIKE '".$word."%'";
					$queries[]  = str_replace( array('{like_query}', '{rmod}'), array($like_query, $rmod), $this->query );

					$like_query = "asp_index.term_reverse LIKE '". (isset($_sr[$wk]) ? $_sr[$wk] : $sr ) ."%'";
					$queries[]  = str_replace( array('{like_query}', '{rmod}'), array($like_query, $rmod), $this->query );
				}
			}

			if (count($queries) > 0) {
				foreach ($queries as $query) {
					$results_arr[] = $wpdb->get_results($query);
				}
			}

			// Merge results depending on the logic
			$results_arr = $this->merge_raw_results($results_arr, $kw_logic);

			// We need to save this array with keys, will need the values later.
			$this->raw_results = $results_arr;

			// Sort results by priority > relevance
			usort($results_arr, array($this, 'compare_by_pr'));

			// Leave only the the LIMIT amount, 500 for non-ajax cases
			if ( isset($options['non_ajax_search']) )
				$results_arr = array_slice($results_arr, 0, 500, true);
			else
				$results_arr = array_slice($results_arr, 0, $searchData['maxresults'], true);

			$this->results = $results_arr;

			// Do some pre-processing
			$this->pre_process_results();

			return $this->results;

		}


		/**
		 * Merges the initial results array, creating an union or intersection.
		 *
		 * The function also adds up the relevance values of the results object.
		 *
		 * @param array $results_arr
		 * @param bool $kw_logic keyword logic (and, or, andex, orex)
		 * @return array results array
		 */
		protected function merge_raw_results($results_arr, $kw_logic = "or") {

			/*
			 * When using the "and" logic, the $results_arr contains the results in [term, term_reverse]
			 * results format. These should not be intersected with each other, so this small code
			 * snippet here divides the results array by groups of 2, then it merges ever pair to one result.
			 * This way it turns into [term1, term1_reverse, term2 ...]  array to [term1 union term1_reversed, ...]
			 *
			 * This is only neccessary with the "and" logic. Others work fine.
			 */
			if ($kw_logic == 'and') {
				$new_ra = array();
				$i = 0;
				$tmp_v = array();
				foreach ($results_arr as $_k => $_v) {
					if ($i & 1){
						// odd, so merge the previous with the current
						$new_ra[] = array_merge($tmp_v, $_v);
					}
					$tmp_v = $_v;
					$i++;
				}
				$results_arr = $new_ra;
			}

			$final_results = array();
			foreach ($results_arr as $results) {
				foreach ($results as $k => $r) {
					if ( isset( $final_results[ $r->blogid . "x" . $r->id ] ) ) {
						$final_results[ $r->blogid . "x" . $r->id ]->relevance += $r->relevance;
					} else {
						$final_results[ $r->blogid . "x" . $r->id ] = $r;
					}
				}
			}

			if ($kw_logic == 'or' || $kw_logic == 'orex')
				return $final_results;

			foreach ($results_arr as $results) {
				$final_results = array_uintersect($final_results, $results, array($this, 'compare_results'));
			}

			return $final_results;
		}


		/**
		 * A custom comparison function for results intersection
		 *
		 * @param $a
		 * @param $b
		 *
		 * @return mixed
		 */
		protected function compare_results($a, $b) {
			if ($a->blogid === $b->blogid)
				return $b->id - $a->id;
			// For different blogids return difference
			return $b->blogid - $a->blogid;
		}

		/**
		 * usort() custom function, sort by ID
		 *
		 * @param $obj_a
		 * @param $obj_b
		 * @return mixed
		 */
		protected function compare_posts($obj_a, $obj_b) {
			return $obj_a->id - $obj_b->id;
		}

		/**
		 * usort() custom function, sort by priority > relevance
		 *
		 * @param $a
		 * @param $b
		 * @return int
		 */
		protected function compare_by_pr($a, $b) {
			if ($a->priority === $b->priority)
				return $b->relevance - $a->relevance;
			return $b->priority - $a->priority;
		}

		private function pre_process_results() {
			// No results, save some resources
			if (count($this->results) == 0)
				return array();

			$pageposts = array();
			$options = $this->options;
			$searchId = $this->searchId;
			$searchData = $this->searchData;
			$post_ids = array();
			//$orderby = w_isset_def($searchData['orderby'], "post_date DESC");
			$the_posts = array();

			// Get the post IDs
			foreach ($this->results as $k => $v) {
				$post_ids[$v->blogid][] = $v->id;
			}

            $_post_types = array();
            if (isset($options['set_inposts']))
                $_post_types[] = 'post';
            if (isset($options['set_inpages']))
                $_post_types[] = 'page';
            if (isset($options['customset']) && is_array($options['customset']))
                $_post_types = array_merge($_post_types, $options['customset']);

			foreach ( $post_ids as $blogid => $the_ids ) {
				if (isset($options['switch_on_preprocess']) && is_multisite())
					switch_to_blog($blogid);
				$args      = array(
					'post__in'       => $the_ids,
					'order_by'       => 'post__in',
					'posts_per_page' => 1000000,
					'post_type'      => $_post_types
				);

				$get_posts = get_posts( $args );
				foreach ($get_posts as $gk=>$gv)
					$get_posts[$gk]->blogid = $blogid;
				$the_posts = array_merge( $the_posts, $get_posts );
			}

			if (isset($options['switch_on_preprocess']) && is_multisite())
				restore_current_blog();


			// Merge the posts with the raw results to a new array
			foreach ($the_posts as $k => $r) {
				$new_result = new stdClass();

				$new_result->id = w_isset_def($r->ID, null);
				$new_result->blogid = $r->blogid;
				$new_result->title = w_isset_def($r->post_title, null);
                $new_result->post_title = $new_result->title;
				$new_result->content = w_isset_def($r->post_content, null);
				$new_result->excerpt = w_isset_def($r->post_excerpt, null);
				$new_result->image = null;


                if ($searchData['showauthor'] == 1) {
                    $post_user = get_user_by("id", $r->post_author);

                    if ( $post_user !== false ) {
                        if ( $searchData['author_field'] == "display_name" )
                            $new_result->author = $post_user->data->display_name;
                        else
                            $new_result->author = $post_user->data->user_login;
                    } else {
                        $new_result->author = null;
                    }
                }


				$new_result->date = w_isset_def($r->post_date, null);
                $new_result->post_date = $new_result->date;

				// Get the relevance and priority values
				$new_result->relevance = (int)$this->raw_results[$new_result->blogid . "x" . $new_result->id]->relevance;
				$new_result->priority = (int)$this->raw_results[$new_result->blogid . "x" . $new_result->id]->priority;
				$new_result->post_type = $this->raw_results[$new_result->blogid . "x" . $new_result->id]->post_type;
				$new_result->content_type = "pagepost";
				$new_result->ttid = $this->raw_results[$new_result->blogid . "x" . $new_result->id]->ttid;

				$pageposts[] = $new_result;
			}

			// Order them once again
			if (count($pageposts) > 0) {

                usort( $pageposts, array( $this, 'compare_by_primary' ) );

                /**
                 * Let us save some time. There is going to be a user selecting the same sorting
                 * for both primary and secondary. Only do secondary if it is different from the primary.
                 */
                if ( $this->ordering['primary'] != $this->ordering['secondary'] ) {
                    $i = 0;
                    foreach ($pageposts as $pk => $pp) {
                        $pageposts[$pk]->primary_order = $i;
                        $i++;
                    }
                    usort( $pageposts, array( $this, 'compare_by_secondary' ) );
                }
			}

			$this->results = $pageposts;
		}
	}
}