<?php
if (!defined('ABSPATH')) die('-1');

if (!class_exists("WD_ASP_Search_Shortcode")) {
    /**
     * Class WD_ASP_Search_Shortcode
     *
     * Search bar shortcode
     *
     * @class         WD_ASP_Search_Shortcode
     * @version       1.0
     * @package       AjaxSearchPro/Classes/Shortcodes
     * @category      Class
     * @author        Ernest Marcinko
     */
    class WD_ASP_Search_Shortcode extends WD_ASP_Shortcode_Abstract {

        /**
         * Overall instance count
         *
         * @var int
         */
        private static $instanceCount = 0;

        /**
         * Used in views, true if the data view is printed
         *
         * @var bool
         */
        private static $dataPrinted = false;

        /**
         * Instance count per search ID
         *
         * @var array
         */
        private static $perInstanceCount = array();

        /**
         * Does the search shortcode stuff
         *
         * @param array|null $atts
         * @return string|void
         */
        function handle($atts) {
            $style = null;

            // Fallback on IE<=8
            if(isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/(?i)msie [6-8]/',$_SERVER['HTTP_USER_AGENT']) ) {
                get_search_form( true );
                return;
            }

            extract(shortcode_atts(array(
                'id' => 'something',
                'extra_class' => ''
            ), $atts));

            if ( WD_ASP_Ajax::doingAjax("ajaxsearchpro_preview") ) {
                require_once(ASP_PATH . "backend" . DIRECTORY_SEPARATOR . "settings" . DIRECTORY_SEPARATOR . "types.inc.php");
                parse_str($_POST['formdata'], $style);
                $style = wpdreams_parse_params($style);
            } else {
                // Visual composer, first selected row, no data, so select the first one
                if ($id == 99999) {
                    $_instances = wd_asp()->instances->get();
                    if ( empty($_instances) )
                        return "There are no search instances to display. Please create one.";

                    $search = reset($_instances);
                } else {
                    $_instance = wd_asp()->instances->get($id);
                    if ( empty($_instance) )
                        return "This search form (with id $id) does not exist!";
                    $search = $_instance;
                }

                // Parse the id again for correction
                $id = $search['id'] + 0;
                $wpdreams_ajaxsearchpros[$id] = 1;
                $style = $search['data'];
            }

            // Don't move this above any return statements!
            self::$instanceCount++;
            if (isset(self::$perInstanceCount[$id]))
                self::$perInstanceCount[$id]++;
            else
                self::$perInstanceCount[$id] = 1;

            $style = array_merge(wd_asp()->options['asp_defaults'], $style);

            // Disabled compact layout if the box is hidden anyways
            if ( $style['box_sett_hide_box'] == 1 ) {
                $style['box_compact_layout'] = 0;
                $style['frontend_search_settings_visible'] = 1;
                $style['show_frontend_search_settings'] = 1;
                $style['frontend_search_settings_position'] = "block";
                $style['resultsposition'] = "block";
                $style['charcount'] = 0;
                $style['trigger_on_facet'] = 1;
            }

            $settingsHidden = w_isset_def($style['show_frontend_search_settings'], 1) != 1 ? true : false;

            $asp_f_items = array();
            if (w_isset_def($style['custom_field_items'], "") != "") {
                $asp_f_items = asp_parse_custom_field_filters( $style['custom_field_items'] );
            }

            // Finally make preview changes after option changes
            if ( WD_ASP_Ajax::doingAjax("ajaxsearchpro_preview") ) {
                ob_start();
                include(ASP_PATH . "/css/style.css.php");
                $out = ob_get_contents();
                ob_end_clean();
                //file_put_contents($file, $out, FILE_TEXT);
                $this->fonts( $style );
                ?>
                <div style='display: none;' id="asp_preview_options"><?php echo base64_encode(serialize($style)); ?></div>
                <style>
                    @import url('<?php echo ASP_URL; ?>css/style.basic.css?r=<?php echo rand(1, 123123123); ?>');
                    <?php echo $out; ?>
                </style>
                <?php
            }

            do_action('asp_layout_before_shortcode', $id);

            $out = "";
            ob_start();
            include(ASP_PATH."includes/views/asp.shortcode.php");
            $out = ob_get_clean();

            do_action('asp_layout_after_shortcode', $id);

            return $out;
        }

        /**
         * Importing fonts does not work correctly it appears.
         * Instead adding the links directly to the header is the best way to go.
         */
        function fonts( $style = "" ) {
            $imports = array();
            $font_sources = array("inputfont", "descfont", "titlefont",
                "authorfont", "datefont", "showmorefont", "groupfont",
                "exsearchincategoriestextfont", "groupbytextfont", "settingsdropfont",
                "prestitlefont", "presdescfont", "pressubtitlefont", "search_text_font");


            if ($style != "") {
                foreach($font_sources as $fs) {
                    if (isset($style["import-".$fs]) && !in_array(trim($style["import-".$fs]), $imports))
                        $imports[] = trim($style["import-".$fs]);
                }
            } else {
                foreach (wd_asp()->instances->get() as $instance) {
                    foreach($font_sources as $fs) {
                        if (isset($instance['data']["import-".$fs]) && !in_array(trim($instance['data']["import-".$fs]), $imports))
                            $imports[] = trim($instance['data']["import-".$fs]);
                    }
                }
            }

            foreach ($imports as $import) {
                $import = trim(str_replace(array("@import url(", ");", "https:", "http:"), "", $import));
                ?>
                <link href='<?php echo $import; ?>' rel='stylesheet' type='text/css'>
            <?php
            }
        }

        // ------------------------------------------------------------
        //   ---------------- SINGLETON SPECIFIC --------------------
        // ------------------------------------------------------------
        public static function getInstance() {
            if ( ! ( self::$_instance instanceof self ) ) {
                self::$_instance = new self();
            }

            return self::$_instance;
        }
    }
}